/* Config.hh. Generated by CMake for ignition-transport10. */

#ifndef IGNITION_TRANSPORT_CONFIG_HH_
#define IGNITION_TRANSPORT_CONFIG_HH_

/* Version number */
#define IGNITION_TRANSPORT_MAJOR_VERSION 10
#define IGNITION_TRANSPORT_MINOR_VERSION 0
#define IGNITION_TRANSPORT_PATCH_VERSION 0

#define IGNITION_TRANSPORT_VERSION "10.0"
#define IGNITION_TRANSPORT_VERSION_FULL "10.0.0"

#define IGNITION_TRANSPORT_VERSION_NAMESPACE v10

#define IGNITION_TRANSPORT_VERSION_HEADER "Ignition transport, version 10.0.0\nCopyright (C) 2017 Open Source Robotics Foundation.\nReleased under the Apache 2.0 License.\n\n"

/* #undef BUILD_TYPE_PROFILE */
/* #undef BUILD_TYPE_DEBUG */
#define BUILD_TYPE_RELEASE 1

/* #undef HAVE_IFADDRS */
/* #undef UBUNTU_FOCAL */

#endif
